/******************************************************************************\
*           Copyright (C) 1999 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* MODULE.NAME... EMIF
* FILENAME...... emif_entersr.c
* DATE CREATED.. Fri 11/22/2002
* PROJECT....... CSL - Chip Support Library
* COMPONENT..... 
* IMPORTS....... 
*------------------------------------------------------------------------------
* HISTORY:
*   CREATED:  11/22/2002 - EMIF self-refresh mode 
*------------------------------------------------------------------------------
* DESCRIPTION: 
*    Performs self-refresh for EMIF connnected to SDRAM
*
\******************************************************************************/
#define _EMIF_MOD_

/****************************************\
* include files
\****************************************/    

#include <csl_emif.h> 

#if ((CHIP_5509) || (CHIP_5509A))
    #include <csl_chip.h>
#endif   
 
#if (_EMIFA_SUPPORT) && (_EMIF_SELFREFRESH_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/

/****************************************\
* EMIF static macro declarations
\****************************************/
/* See cache.h Romability */
 
 
/****************************************\
* EMIF static typedef declarations
\****************************************/

/****************************************\
* EMIF static function declarations
\****************************************/

/****************************************\
* EMIF Variable definitions : ROMability 
\****************************************/

/* Definition Romability See emif.h */


             
/****************************************\
* EMIF static function definitions
\****************************************/

/*----------------------------------------------------------------------------*/

/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/

/****************************************\
* EMIF global variable definitions
\****************************************/
/* Definition See emif.h */
/*----------------------------------------------------------------------------*/

             
/****************************************\
* EMIF global function definitions
\****************************************/

#pragma CODE_SECTION(EMIF_enterSelfRefresh,".text:EMIF_enterSelfRefresh")

/*----------------------------------------------------------------------------*/
void EMIF_enterSelfRefresh(Uint16 ckePin, Uint16 tRasDelay) {

#if (CHIP_5509A)

  int oldgie;
  Uint16 preChargeCnt;
  Uint16 i;


  oldgie = IRQ_globalDisable();


          /*    SDRAM self refresh procedure starts here    */

          /*  1.	Software selects the pin to be used for CKE: GPIO4 */

          CHIP_FSET(XBSR,CKE,ckePin);

      /*  2.	Software sets Bit # 7 of Bus Selection Register to 1 (CKE En=1)
        and Bit #6 to 0 (SR Com=0).SR Com must be zero until setup is completed.*/

         CHIP_FSET(XBSR,CKEEN,EMIF_CKEEN_ENABLED);

/*  3.	No accesses to external memory are permitted beyond this step.*/
    
/*  4.	Software needs to wait for a pre-charge command. Pre-charge command is 
        performed when doing an auto-refresh. To wait for a pre-charge command,
        software polls the SDRAM Counter Register and detect when this register
        counts down to 0 and is reloaded with the Period Register. When this 
        change occurs, EMIF sends a precharge command with A10 high to precharge
        all banks.    */
   
        while(EMIF_RGET(SDCNT));
        while(EMIF_RGET(SDCNT) != EMIF_RGET(SDPER));

/*  5.	Wait for the auto refresh to complete (30 clock cycles).      */

        preChargeCnt = 0x30 * ( 1 << EMIF_FGET(EGCR,MEMFREQ));
        while (--preChargeCnt){
            asm("\tNOP      ; Code Auto-Generated by CSL");
        }
        
  
/*  6.	Software sets RFEN= 0 deactivating auto-refresh mode in the EMIF     */
  
        EMIF_FSET(SDC1,RFEN,EMIF_SDC1_RFEN_DISABLE);

  
    
/*  7.	Software sets SR Com=1.*/
    
        CHIP_FSET(XBSR,SRCOM,EMIF_SRCOM_ENABLE);
    
/* Wait for minimum time in refresh */

       for(i=0;i<=tRasDelay-1; i++){
           asm("\tNOP     ; Code Generated by CSL");
       }       

  IRQ_globalRestore(oldgie);

 #endif

}
/*----------------------------------------------------------------------------*/

#endif /* EMIF_SUPPORT */
/******************************************************************************\
* End of emif_entersr.c
\******************************************************************************/
