/******************************************************************************\
*           Copyright (C) 1999 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* MODULE.NAME... EMIF
* FILENAME...... emif_exitsr.c
* DATE CREATED.. Fri 11/22/2002
* PROJECT....... CSL - Chip Support Library
* COMPONENT..... 
* IMPORTS....... 
*------------------------------------------------------------------------------
* HISTORY:
*   CREATED:  11/22/2002 - EMIF self-refresh mode 
*------------------------------------------------------------------------------
* DESCRIPTION: 
*    Performs self-refresh for EMIF connnected to SDRAM
*
\******************************************************************************/
#define _EMIF_MOD_

/****************************************\
* include files
\****************************************/    

#include <csl_emif.h> 
#include <csl_pwr.h>

#if ((CHIP_5509) || (CHIP_5509A))
    #include <csl_chip.h>
#endif   
 
#if (_EMIFA_SUPPORT) && (_EMIF_SELFREFRESH_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/

/****************************************\
* EMIF static macro declarations
\****************************************/
/* See cache.h Romability */
 
 
/****************************************\
* EMIF static typedef declarations
\****************************************/

/****************************************\
* EMIF static function declarations
\****************************************/

/****************************************\
* EMIF Variable definitions : ROMability 
\****************************************/

/* Definition Romability See emif.h */


             
/****************************************\
* EMIF static function definitions
\****************************************/

/*----------------------------------------------------------------------------*/

/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/

/****************************************\
* EMIF global variable definitions
\****************************************/
/* Definition See emif.h */
/*----------------------------------------------------------------------------*/

             
/****************************************\
* EMIF global function definitions
\****************************************/

#pragma CODE_SECTION(EMIF_exitSelfRefresh,".text:EMIF_exitSelfRefresh")

void EMIF_exitSelfRefresh(Uint16 tXsrDelay) {

#if (CHIP_5509A)

 int oldgie;
 Uint16 i;


/* Only execute function if EMIF is not IDLEed */
   
    if ((_PWR_ISTR & _PWR_ISTR_EMIFIS_MASK) != 0) {
        return;
    }

   oldgie = IRQ_globalDisable();

/*  Exiting self refresh mode */

    
/* ENABLE CLKMEM */

     EMIF_FSET(EGCR,MEMCEN,EMIF_EGCR_MEMCEN_ENABLED);
     
/*  1.	Software sets SR Com=0,  disabling self refresh command and driving XF high.*/

     CHIP_FSET(XBSR,SRCOM,EMIF_SRCOM_DISABLED);

/*  2.	Wait until tXSR for a on-going self refresh to complete (around 100ns).*/
     
     for(i=0; i<=tXsrDelay; i++) {
       asm("\tNOP         ; Code Auto-Generated by CSL");
     }
/*  3.	Software sets RFEN= 1, exiting self refresh mode and entering auto refresh mode.   */

     EMIF_FSET(SDC1,RFEN,EMIF_SDC1_RFEN_ENABLE);

/*  4.	Write to EMIFF Initialization Register to re-initialize the EMIF. */

    EMIF_RSET(SDINIT,0x0000u);
    
        while(EMIF_RGET(SDCNT));
        while(EMIF_RGET(SDCNT) != EMIF_RGET(SDPER));

  IRQ_globalRestore(oldgie);
 
 #endif

}
/*----------------------------------------------------------------------------*/

#endif /* EMIF_SUPPORT */
/******************************************************************************\
* End of emif_exitsr.c
\******************************************************************************/
