/******************************************************************************\
*           Copyright (C) 1999 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* MODULE.NAME... TIMER
* FILENAME...... TIM_open.c
* DATE CREATED.. Sun 06/20/1999 
* PROJECT....... Chip Support Library
* COMPONENT..... 
* IMPORTS....... 
*------------------------------------------------------------------------------
* HISTORY:
*   CREATED:  06/20/1999 Created 
*   MODIFIED: 12/06/2000 Modification for C54x .
*   MODIFIED: 02/18/2000 C54x Romability ( All static variable were removed) .
*   MODIFIED: 05/30/2000 Created for C55x .
*   MODIFIED: 06/28/2001 updated for new CSL initialization model and
*                        included pragma for individual section per function
*   MODIFIED: 08/09/2004 Fixed implementaion error in GPT_open()                   
*   MODIFIED: 08/24/2004 Removed IRQ_restoreGie() from 55x CSL
*------------------------------------------------------------------------------
* DESCRIPTION:  (body file for the TIMER module)
*
*
*
\******************************************************************************/
#define _GPT_MOD_

/****************************************\
* include files
\****************************************/    

#include "csl_gpt.h"
   
 
#if (_GPT_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/

/****************************************\
* GPT static macro declarations
\****************************************/
/* See csl_gpt.h Romability */


/*----------------------------------------------------------------------------*/

/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/

/****************************************\
* GPT global variable definitions
\****************************************/
/* Definition See csl_gpt.h */
/*----------------------------------------------------------------------------*/
/* DSP/BIOS symbols that CSL needs to check to see what if any timer devices  */
/* are in use by BIOS. These symbols will e defined in the BIOS linker        */
/* command file generated by gconf.                                           */
/* See C6xx */
             
/****************************************\
* TIMER global function definitions
\****************************************/

#pragma CODE_SECTION(GPT_open,".text:GPT_open")

/*----------------------------------------------------------------------------*/
GPT_Handle GPT_open(int DevNum, Uint32 Flags) {

  GPT_Handle hGpt = (GPT_Handle)INV;
  Uint16 dev  = DevNum;
  int oldgie;

  oldgie = IRQ_globalDisable();  
 
 
  #ifdef _MCRTE_DEBUG
  if ( !((DevNum == -1) || ((DevNum >= 0) && (DevNum < GPT_DEVICE_CNT))) ) {
     ERR_submit(GPT_ERR_MAJOR, GPT_ERR_ALLOC);
     IRQ_globalRestore(oldgie);
    return (GPT_Handle) INV;
  }
  #endif

  if (DevNum==-1) {
    for (dev=0; dev<GPT_DEVICE_CNT; dev++) {
      if (!(CSL_SYS_DATA.GptAllocMask & (1u<<dev))){
        hGpt = GPT_hDev(dev);
       break;
      }
    }
  } else if (!(CSL_SYS_DATA.GptAllocMask & (1u<<dev))){
    hGpt = GPT_hDev(dev);
  }

  #ifdef _MCRTE_DEBUG
    if (hGpt == INV) {
      ERR_submit(GPT_ERR_MAJOR, GPT_ERR_INVALID_HANDLE);
      IRQ_globalRestore(oldgie);
      return (GPT_Handle)INV;
  }
  #endif

  if (hGpt != INV) {
     CSL_SYS_DATA.GptAllocMask |= 1u<<dev;     
     if (Flags & GPT_OPEN_RESET) {
       GPT_reset(hGpt);
     }
  }

  IRQ_globalRestore(oldgie);
  return hGpt;
}

/*----------------------------------------------------------------------------*/

#endif /* GPT_SUPPORT */
/******************************************************************************\
* End of gpt_open.c
\******************************************************************************/

