/******************************************************************************\
*           Copyright (C) 1999 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* MODULE.NAME... TIMER
* FILENAME...... TIM_open.c
* DATE CREATED.. Sun 06/20/1999 
* PROJECT....... Chip Support Library
* COMPONENT..... 
* IMPORTS....... 
*------------------------------------------------------------------------------
* HISTORY:
*   CREATED:  06/20/1999 Created 
*   MODIFIED: 12/06/2000 Modification for C54x .
*   MODIFIED: 02/18/2000 C54x Romability ( All static variable were removed) .
*   MODIFIED: 05/30/2000 Created for C55x .
*   MODIFIED: 06/28/2001 updated for new CSL initialization model and
*                        included pragma for individual section per function
*   MODIFIED: 07/22/2003 Fixed the typo 'Timer_open' in the CODE_SECTION pragma.
*   MODIFIED: 08/09/2004 Fixed implementation error in TIMER_open()
*   MODIFIED: 08/24/2004 Removed IRQ_restoreGie() from 55x CSL
*------------------------------------------------------------------------------
* DESCRIPTION:  (body file for the TIMER module)
*
*
*
\******************************************************************************/
#define _TIMER_MOD_

/****************************************\
* include files
\****************************************/    

#include <csl_chiphal.h>
 
#if (_TIMER_SUPPORT)

#include <csl_timer.h>
   
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/

/****************************************\
* TIMER static macro declarations
\****************************************/
/* See timer.h Romability */

#define ALLOCATED(hTimer) (((TIMER_PrivateObj *)hTimer)->Allocated)
#define EVENTID(hTimer)   (((TIMER_PrivateObj*)hTimer)->EventId)  
#define TCR(hTimer) PREG16(((TIMER_PrivateObj*)hTimer)->TcrAddr)
#define PRD(hTimer) PREG16(((TIMER_PrivateObj*)hTimer)->PrdAddr)
#define TIM(hTimer) PREG16(((TIMER_PrivateObj*)hTimer)->TimAddr) 
#define PRSC(hTimer) PREG16(((TIMER_PrivateObj*)hTimer)->PrscAddr) 
 
 
/****************************************\
* TIMER static typedef declarations
\****************************************/

/****************************************\
* TIMER static function declarations
\****************************************/

/****************************************\
* TIMER Variable definitions : ROMability 
\****************************************/

/* Definition Romability See timer.h */


             
/****************************************\
* TIMER static function definitions
\****************************************/

/*----------------------------------------------------------------------------*/

/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/

/****************************************\
* TIMER global variable definitions
\****************************************/
/* Definition See timer.h */
/*----------------------------------------------------------------------------*/
/* DSP/BIOS symbols that CSL needs to check to see what if any timer devices  */
/* are in use by BIOS. These symbols will e defined in the BIOS linker        */
/* command file generated by gconf.                                           */
/* See C6xx */
             
/****************************************\
* TIMER global function definitions
\****************************************/

#pragma CODE_SECTION(TIMER_open,".text:TIMER_open")

/*----------------------------------------------------------------------------*/
TIMER_Handle TIMER_open(int DevNum, Uint16 Flags) {

  TIMER_Handle hTimer = INV;
  Uint16 dev  = DevNum;
  int oldgie;

  oldgie = IRQ_globalDisable();  
 
 
  #ifdef _MCRTE_DEBUG
  
  if ( !((DevNum == -1) || ((DevNum >= 0) && (DevNum < TIMER_DEVICE_CNT))) ) {
     ERR_submit(TIMER_ERR_MAJOR, TIMER_ERR_ALLOC);
     IRQ_globalRestore(oldgie);
    return (TIMER_Handle) INV;
  }
  #endif

  if (DevNum==-1) {
    for (dev=0; dev<TIMER_DEVICE_CNT; dev++) {
      if (!(CSL_SYS_DATA.TimerAllocMask & (1u<<dev))){
        hTimer = TIMER_hDev(dev);
       break;
      }
    }
  } else if (!(CSL_SYS_DATA.TimerAllocMask & (1u<<dev))){
    hTimer = TIMER_hDev(dev);
  }

  #ifdef _MCRTE_DEBUG
    if (hTimer == INV) {
      ERR_submit(TIMER_ERR_MAJOR, TIMER_ERR_INVALID_HANDLE);
      IRQ_globalRestore(oldgie);
      return (TIMER_Handle)INV;
  }
  #endif

  if (hTimer != INV) {
     CSL_SYS_DATA.TimerAllocMask |= 1u<<dev;     
     if (Flags & TIMER_OPEN_RESET) {
       TIMER_reset(hTimer);
     }
  }

  IRQ_globalRestore(oldgie);
  return hTimer;
}

/*----------------------------------------------------------------------------*/

#endif /* TIMER_SUPPORT */
/******************************************************************************\
* End of TIM_open.c
\******************************************************************************/

