/******************************************************************************\
*           Copyright (C) 1999 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* MODULE.NAME... TIMER
* FILENAME...... tim_rst.c
* DATE CREATED.. Sun 06/20/1999 
* PROJECT....... Chip Support Library
* COMPONENT..... Service Layer
* IMPORTS....... 
*------------------------------------------------------------------------------
* HISTORY:
*   CREATED:  06/20/1999
*   MODIFIED: 12/06/2000 Modification for C54x .
*   MODIFIED: 02/18/2000 C54x Romability ( All static variable were removed) .
*   MODIFIED: 05/30/2000 (NNS) Created for C55x .
*   MODIFIED: 06/28/2001 included pragma for creation of code section per
*              function
*------------------------------------------------------------------------------
* DESCRIPTION:  (body file for the TIMER module)
*
*
*
\******************************************************************************/
#define _TIMER_MOD_

/****************************************\
* include files
\****************************************/
    

   
#include <csl_chiphal.h>  
 
#if (_TIMER_SUPPORT)

#include <csl_timer.h>

/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/

/****************************************\
* TIMER static macro declarations
\****************************************/
/* See timer.h Romability */

#define ALLOCATED(hTimer) (((TIMER_PrivateObj *)hTimer)->Allocated)
#define EVENTID(hTimer)   (((TIMER_PrivateObj*)hTimer)->EventId)  
#define TCR(hTimer)       PREG16(((TIMER_PrivateObj*)hTimer)->TcrAddr)
#define PRD(hTimer)       PREG16(((TIMER_PrivateObj*)hTimer)->PrdAddr)
#define TIM(hTimer)       PREG16(((TIMER_PrivateObj*)hTimer)->TimAddr) 
#define PRSC(hTimer)      PREG16(((TIMER_PrivateObj*)hTimer)->PrscAddr)  
 
/****************************************\
* TIMER static typedef declarations
\****************************************/

/****************************************\
* TIMER static function declarations
\****************************************/

/****************************************\
* TIMER Variable definitions : ROMability 
\****************************************/

/* Definition Romability See timer.h */


             
/****************************************\
* TIMER static function definitions
\****************************************/

/*----------------------------------------------------------------------------*/

/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/

/****************************************\
* TIMER global variable definitions
\****************************************/
/* Definition See timer.h */
/*----------------------------------------------------------------------------*/
/* DSP/BIOS symbols that CSL needs to check to see what if any timer devices  */
/* are in use by BIOS. These symbols will e defined in the BIOS linker        */
/* command file generated by gconf.                                           */
/* See C6xx */
             
/****************************************\
* TIMER global function definitions
\****************************************/

#pragma CODE_SECTION(TIMER_reset,".text:TIMER_reset")

/*----------------------------------------------------------------------------*/
void TIMER_reset(TIMER_Handle hTimer) {
  int oldgie;
  Uint16 EventId;

  oldgie = IRQ_globalDisable();  
    
  if (hTimer == INV) {    
 
      TIMER_reset(TIMER_hDev0);
      TIMER_reset(TIMER_hDev1);  
       
  } 
  else {
      EventId  = TIMER_getEventId(hTimer);
      /* stop timer */  
      TIMER_FSETH(hTimer,TCR,TSS,1); 
      /* Reset Period Value=0xFFFF */
      TIMER_RSETH(hTimer,PRD,0xFFFF);
      TIMER_FSETH(hTimer,TCR,TLB,1);
      /* Reset Bit Value */
      TIMER_RSETH(hTimer,TCR,TIMER_TCR_DEFAULT);
      IRQ_disable(EventId);
      IRQ_clear(EventId);
  }
  IRQ_globalRestore(oldgie);

}

/*----------------------------------------------------------------------------*/

#endif /* TIMER_SUPPORT */
/******************************************************************************\
* End of tim_rst.c
\******************************************************************************/

